* define coefficients

$include RTsim_input.gms
* select a cost curve setting
* avaiable: 1, 2, 4, 8, 16, 40, 160
set      j energy segments /j1/;
scalar   i index /1/;
set      tt price response horizon /tt1*tt24/;

parameter
         C_es(j)         BES cost curve
         soc_max(b,j)    BES energy segments
         C_ch(j)         BES discharge cost curve
         C_dis(j)        BES charge cost curve
;

if(card(j) eq 1,
         C_es('j1') = C_p001_table('jj1','col1');
elseif card(j) eq 2,
         while(i le card(j),
                 C_es(j) = sum((jj,col)$(ord(jj) eq ord(j)), C_p002_table(jj,col));
                 i=i+1;
         );
elseif card(j) eq 4,
         while(i le card(j),
                 C_es(j) = sum((jj,col)$(ord(jj) eq ord(j)), C_p004_table(jj,col));
                 i=i+1;
         );
elseif card(j) eq 8,
         while(i le card(j),
                 C_es(j) = sum((jj,col)$(ord(jj) eq ord(j)), C_p008_table(jj,col));
                 i=i+1;
         );
elseif card(j) eq 16,
         while(i le card(j),
                 C_es(j) = sum((jj,col)$(ord(jj) eq ord(j)), C_p016_table(jj,col));
                 i=i+1;
         );
elseif card(j) eq 40,
         while(i le card(j),
                 C_es(j) = sum((jj,col)$(ord(jj) eq ord(j)), C_p040_table(jj,col));
                 i=i+1;
         );
elseif card(j) eq 80,
         while(i le card(j),
                 C_es(j) = sum((jj,col)$(ord(jj) eq ord(j)), C_p080_table(jj,col));
                 i=i+1;
         );
elseif card(j) eq 160,
         while(i le card(j),
                 C_es(j) = sum((jj,col)$(ord(jj) eq ord(j)), C_p160_table(jj,col));
                 i=i+1;
         );
);

* parameters
scalar
         s_num   small number /1e-6/
         eta     charging and discharging efficiency /0.95/
         Pes     energy storage rated power MW /1/
         Ees     energy storage rated capacity MWh /1/
         Pesmin  energy storage minimum power /.1/
         SoCmin  minimum soc /0.15/
         SoCmax  maximum soc /0.95/
         Crep    energy storage cell replacement cost $-MWh /300e3/
         H       reserve continuous output requirement /1/
;

* set marginal costsw
C_ch(j) = 0;
C_dis(j) = Crep * C_es(j) /eta;
soc_max(b,j) = (SoCmax-SoCmin)/card(j);


* define variables
variables
         ES_revenue      storage revenue
         rev(b)          storage revenue
;

positive variables
         ch(tt,b,j)       charge power
         dis(tt,b,j)      discharge power
         soc(tt,b,j)      soc
         q(tt,b)          tmsr
;

binary variables
         v_es(tt,b)       charging discharging status
;

* Define parameters
parameters
         lmp(tt,b)       lmp
         tmsr(tt,b)      tmsr
         soc0(b,j)       initial soc
;

equations
         obj            objective function
         es0(b)         revenue each es
         es1(tt,b)       energy storage charging power constraint
         es2(tt,b)       energy storage discharging power constraint
         es3(tt,b,j)     energy storage soc evolution
         es4(tt,b,j)     energy storage soc evolution initial
         es5(tt,b,j)     maximum soc constraint
         es6(tt,b)       reserve power requirement
         es7(tt,b)       reserve energy requirement
;

obj..
         ES_revenue =e= sum(b, rev(b))
;

es0(b)..
         rev(b) =e= sum((tt,j), lmp(tt,b)*(dis(tt,b,j)-ch(tt,b,j))
         + tmsr(tt,b)*q(tt,b)
         - C_dis(j)*dis(tt,b,j)- C_ch(j)*ch(tt,b,j))
;

es1(tt,b)..
         sum(j, ch(tt,b,j)) =l= Pes * (1-v_es(tt,b))
;

es2(tt,b)..
         sum(j, dis(tt,b,j)) =l= Pes * v_es(tt,b)
;

es3(tt,b,j)$(ord(tt) gt 1)..
         soc(tt,b,j) - soc(tt-1,b,j) =e= ch(tt,b,j)*eta - dis(tt,b,j)/eta
;

es4(tt,b,j)$(ord(tt) eq 1)..
         soc(tt,b,j) - soc0(b,j) =e= ch(tt,b,j)*eta - dis(tt,b,j)/eta
;

es5(tt,b,j)..
         soc(tt,b,j) =l= soc_max(b,j)
;

es6(tt,b)..
         sum(j, dis(tt,b,j)) + q(tt,b) - sum(j, ch(tt,b,j)) =l= Pes
;

es7(tt,b)..
         sum(j,soc(tt,b,j)) - q(tt,b) =g= 0
;

* Start simulation

model isone_es_2015  /all/;

scalar counter /1/;

soc0(b,j) = 0;

parameter
         lmp_daily(e,t,b)       lmp
         es_cost_total(b)        es cost
         es_cost_day(e,b)        es cost
         es_revenue_total(b)         total revenue
         es_revenue_day(e,b)       daily revenue
         soc_day(e,t,b)  daily soc
         ch_day(e,t,b)   charge
         dis_day(e,t,b)   discharge
         es_rev_lmp(e,b)
         es_rev_tmsr(e,b)
;

while( counter le card(e),
         lmp(tt,b)$(ord(tt) le card(t)) = sum((e,t)$(ord(e) eq counter and ord(tt) eq ord(t)),
                         lmp2015_rt(e,t,b));
         if( counter lt card(e),
                 lmp(tt,b)$(ord(tt) gt card(t)) = sum((e,t)$(ord(e) eq counter+1 and ord(tt) eq (ord(t)+card(t))),
                         lmp2015_rt(e,t,b));
         else
                 lmp(tt,b)$(ord(tt) gt card(t)) = 0;
         );
         tmsr(tt,b)$(ord(tt) le card(t)) = sum((e,t)$(ord(e) eq counter and ord(tt) eq ord(t)),
                         tmsr_2015(e,t,b));
         if( counter lt card(e),
                 tmsr(tt,b)$(ord(tt) gt card(t)) = sum((e,t)$(ord(e) eq counter+1 and ord(tt) eq (ord(t)+card(t))),
                         tmsr_2015(e,t,b));
         else
                 tmsr(tt,b)$(ord(tt) gt card(t)) = 0;
         );

*         tmsr(tt,b) = 0;
         solve isone_es_2015 using mip maximizing ES_revenue;

         lmp_daily(e,t,b)$(ord(e) eq counter) = sum(tt$(ord(tt) eq ord(t)),lmp(tt,b)) + s_num;
*         es_cost_day(e,b)$(ord(e) eq counter) = sum((t,j), C_dis(j)*dis.l(t,b,j) +  C_ch(j)*ch.l(t,b,j)) + s_num;
*         es_revenue_day(e,b)$(ord(e) eq counter) = rev.l(b) + s_num;
         soc_day(e,t,b)$(ord(e) eq counter) = sum(tt$(ord(tt) eq ord(t)),sum(j,soc.l(tt,b,j)))
                                                  + SoCmin + s_num;
         ch_day(e,t,b)$(ord(e) eq counter) = sum(tt$(ord(tt) eq ord(t)),sum(j,ch.l(tt,b,j))) + s_num;
         dis_day(e,t,b)$(ord(e) eq counter) = sum(tt$(ord(tt) eq ord(t)),sum(j,dis.l(tt,b,j))) + s_num;

         es_cost_day(e,b)$(ord(e) eq counter) = sum((tt,j)$(ord(tt) le card(t)), C_dis(j)*dis.l(tt,b,j) +  C_ch(j)*ch.l(tt,b,j)) + s_num;
         es_revenue_day(e,b)$(ord(e) eq counter) = sum((tt,j)$(ord(tt) le card(t)), lmp(tt,b)*(dis.l(tt,b,j)-ch.l(tt,b,j))) + s_num;
         es_rev_tmsr(e,b)$(ord(e) eq counter) = sum((tt)$(ord(tt) le card(t)), tmsr(tt,b)*q.l(tt,b)) + s_num;

         soc0(b,j) = soc.l('tt24',b,j);

         counter = counter+1;

*         display lmp, ch.l, dis.l, soc.l, ch_day, dis_day, es_cost_day;
);

es_revenue_total(b) = sum(e, es_revenue_day(e,b) + es_rev_tmsr(e,b) - 2*s_num);
es_cost_total(b) = sum(e, es_cost_day(e,b) - s_num);

Execute_Unload 'ESRT2015_Seg000'
         lmp_daily,
         es_revenue_total,
         es_cost_total,
         es_revenue_day,
         es_rev_tmsr,
         es_cost_day,
         soc_max, C_es, eta, Crep,
         soc_day, ch_day, dis_day
;
